@@ -31,6 +31,7 @@
 #include <unistd.h>
 #include <time.h>
 
+#include <camel/camel.h>
 #include <libedataserver/e-sexp.h>
 #include <libedataserver/e-uid.h>
 #include <libebook/e-address-western.h>
@@ -42,10 +43,6 @@
 #include <libedata-book/e-book-backend-cache.h>
 #include <libedataserver/e-xml-hash-utils.h>
 
-#include <camel/camel-mime-message.h>
-#include <camel/camel-multipart.h>
-#include <camel/camel-stream-mem.h>
-
 #include <e2k-context.h>
 #include <e2k-propnames.h>
 #include <e2k-restriction.h>
@@ -492,7 +489,7 @@ e_contact_from_props (EBookBackendExchange *be, E2kResult *result)
 				e_vcard_attribute_add_value (attr, value);
 
 			g_free (value);
-			camel_object_unref (addr);
+			g_object_unref (addr);
 
 			e_vcard_attribute_add_param_with_value (attr,
 					e_vcard_attribute_param_new (EEX_X_MEMBERID),
@@ -524,9 +521,9 @@ e_contact_from_props (EBookBackendExchange *be, E2kResult *result)
 	soup_buffer_free (response);
 	msg = camel_mime_message_new ();
 	camel_data_wrapper_construct_from_stream (CAMEL_DATA_WRAPPER (msg), stream);
-	camel_object_unref (stream);
+	g_object_unref (stream);
 
-	content = camel_medium_get_content_object (CAMEL_MEDIUM (msg));
+	content = camel_medium_get_content (CAMEL_MEDIUM (msg));
 	if (CAMEL_IS_MULTIPART (content)) {
 		multipart = (CamelMultipart *)content;
 		content = NULL;
@@ -535,30 +532,30 @@ e_contact_from_props (EBookBackendExchange *be, E2kResult *result)
 			part = camel_multipart_get_part (multipart, i);
 			filename = camel_mime_part_get_filename (part);
 			if (filename && !strncmp (filename, "ContactPicture.", 15)) {
-				content = camel_medium_get_content_object (CAMEL_MEDIUM (part));
+				content = camel_medium_get_content (CAMEL_MEDIUM (part));
 				break;
 			}
 		}
 
 		if (content) {
 			EContactPhoto photo;
-			GByteArray *mem_bytes;
+			GByteArray *byte_array;
 
-			mem_bytes = g_byte_array_new ();
-			stream = camel_stream_mem_new_with_byte_array (mem_bytes);
+			byte_array = g_byte_array_new ();
+			stream = camel_stream_mem_new_with_byte_array (byte_array);
 			camel_data_wrapper_decode_to_stream (content, stream);
 
 			photo.type = E_CONTACT_PHOTO_TYPE_INLINED;
 			photo.data.inlined.mime_type = NULL;
-			photo.data.inlined.data = mem_bytes->data;
-			photo.data.inlined.length = mem_bytes->len;
+			photo.data.inlined.data = byte_array->data;
+			photo.data.inlined.length = byte_array->len;
 			e_contact_set (contact, E_CONTACT_PHOTO, &photo);
 
-			camel_object_unref (stream);
+			g_object_unref (stream);
 		}
 	}
 
-	camel_object_unref (msg);
+	g_object_unref (msg);
 	return contact;
 }
 
@@ -1025,7 +1022,7 @@ populate_date(EContactField field, EContact *new_contact, gpointer data)
 
 	dt.year = then->tm_year + 1900;
 	dt.month = then->tm_mon + 1;
-	dt.day = then->tm_mday;
+	dt.day = then->tm_mday + 1;
 
 	e_contact_set (new_contact, field, &dt);
 }
@@ -1360,14 +1357,14 @@ build_message (const gchar *from_name, const gchar *from_email,
 	from = camel_internet_address_new ();
 	camel_internet_address_add (from, from_name, from_email);
 	camel_mime_message_set_from (msg, from);
-	camel_object_unref (from);
+	g_object_unref (from);
 
 	/* Create the body */
 	if (note) {
 		stream = camel_stream_mem_new_with_buffer (note, strlen (note));
 		wrapper = camel_data_wrapper_new ();
 		camel_data_wrapper_construct_from_stream (wrapper, stream);
-		camel_object_unref (stream);
+		g_object_unref (stream);
 
 		type = camel_content_type_new ("text", "plain");
 		camel_content_type_set_param (type, "charset", "UTF-8");
@@ -1382,7 +1379,7 @@ build_message (const gchar *from_name, const gchar *from_email,
 		text_part = CAMEL_MIME_PART (msg);
 
 	if (text_part) {
-		camel_medium_set_content_object (CAMEL_MEDIUM (text_part), wrapper);
+		camel_medium_set_content (CAMEL_MEDIUM (text_part), wrapper);
 		camel_mime_part_set_encoding (text_part, CAMEL_TRANSFER_ENCODING_8BIT);
 	}
 	if (photo) {
@@ -1421,11 +1418,11 @@ build_message (const gchar *from_name, const gchar *from_email,
 
 		wrapper = camel_data_wrapper_new ();
 		camel_data_wrapper_construct_from_stream (wrapper, stream);
-		camel_object_unref (stream);
+		g_object_unref (stream);
 		camel_data_wrapper_set_mime_type (wrapper, content_type);
 
 		photo_part = camel_mime_part_new ();
-		camel_medium_set_content_object (CAMEL_MEDIUM (photo_part),
+		camel_medium_set_content (CAMEL_MEDIUM (photo_part),
 						 wrapper);
 		camel_mime_part_set_encoding (photo_part, CAMEL_TRANSFER_ENCODING_BASE64);
 		camel_mime_part_set_description (photo_part, filename);
@@ -1438,22 +1435,22 @@ build_message (const gchar *from_name, const gchar *from_email,
 		camel_multipart_set_boundary (multipart, NULL);
 		if (text_part) {
 			camel_multipart_add_part (multipart, text_part);
-			camel_object_unref (text_part);
+			g_object_unref (text_part);
 		}
 		camel_multipart_add_part (multipart, photo_part);
-		camel_object_unref (photo_part);
+		g_object_unref (photo_part);
 
-		camel_medium_set_content_object (CAMEL_MEDIUM (msg),
+		camel_medium_set_content (CAMEL_MEDIUM (msg),
 						 CAMEL_DATA_WRAPPER (multipart));
-		camel_object_unref (multipart);
+		g_object_unref (multipart);
 	}
 
 	buffer = g_byte_array_new();
 	stream = camel_stream_mem_new ();
 	camel_stream_mem_set_byte_array (CAMEL_STREAM_MEM (stream), buffer);
 	camel_data_wrapper_write_to_stream (CAMEL_DATA_WRAPPER (msg), stream);
-	camel_object_unref (stream);
-	camel_object_unref (msg);
+	g_object_unref (stream);
+	g_object_unref (msg);
 
 	return buffer;
 }
@@ -1624,7 +1621,7 @@ merge_contact_lists (EBookBackendExchange *be, const gchar *location, EContact *
 				}
 			}
 		}
-		camel_object_unref (addr);
+		g_object_unref (addr);
 	}
 
 	/* remove all the members from the server which left - they has been removed during editing probably */
