@@ -34,10 +34,7 @@
 #include <gio/gio.h>
 #include <glib/gstdio.h>
 
-#include <camel/camel-mime-message.h>
-#include <camel/camel-multipart.h>
-#include <camel/camel-stream-mem.h>
-#include <camel/camel-file-utils.h>
+#include <camel/camel.h>
 
 #include "e-cal-backend-exchange.h"
 #include "e2k-cal-utils.h"
@@ -1717,9 +1714,9 @@ get_attachment (ECalBackendExchange *cbex, const gchar *uid,
 	stream = camel_stream_mem_new_with_buffer (body, len);
 	msg = camel_mime_message_new ();
 	camel_data_wrapper_construct_from_stream (CAMEL_DATA_WRAPPER (msg), stream);
-	camel_object_unref (stream);
+	g_object_unref (stream);
 
-	msg_content = camel_medium_get_content_object (CAMEL_MEDIUM (msg));
+	msg_content = camel_medium_get_content (CAMEL_MEDIUM (msg));
 	if (msg_content && CAMEL_IS_MULTIPART (msg_content)) {
 		multipart = (CamelMultipart *)msg_content;
 
@@ -1727,29 +1724,28 @@ get_attachment (ECalBackendExchange *cbex, const gchar *uid,
 			part = camel_multipart_get_part (multipart, i);
 			filename = camel_mime_part_get_filename (part);
 			if (filename) {
-				CamelStreamMem *stream_mem;
+				GByteArray *byte_array;
 
-				content = camel_medium_get_content_object (CAMEL_MEDIUM (part));
-
-				stream = camel_stream_mem_new ();
-				stream_mem = (CamelStreamMem *)stream;
+				content = camel_medium_get_content (CAMEL_MEDIUM (part));
 
+				byte_array = g_byte_array_new ();
+				stream = camel_stream_mem_new_with_byte_array (byte_array);
 				camel_data_wrapper_decode_to_stream (content, stream);
-				attach_data = g_memdup (stream_mem->buffer->data, stream_mem->buffer->len);
+				attach_data = g_memdup (byte_array->data, byte_array->len);
 				attach_file = g_strdup_printf ("%s/%s-%s", cbex->priv->local_attachment_store, uid, filename);
 				// Attach
-				attach_file_url = save_attach_file (attach_file, (gchar *) attach_data, stream_mem->buffer->len);
+				attach_file_url = save_attach_file (attach_file, (gchar *) attach_data, byte_array->len);
 				g_free (attach_data);
 				g_free (attach_file);
 				d(printf ("attach file name : %s\n", attach_file_url));
 				list = g_slist_append (list, attach_file_url);
 
-				camel_object_unref (stream);
+				g_object_unref (stream);
 			}
 		} /* Loop through each multipart */
 	}
 
-	camel_object_unref (msg);
+	g_object_unref (msg);
 	return list;
 }
 
@@ -1935,13 +1931,13 @@ gchar *
 build_msg ( ECalBackendExchange *cbex, ECalComponent *comp, const gchar *subject, gchar **boundary)
 {
 	CamelMimeMessage *msg;
-	CamelStreamMem *content;
 	CamelMimePart *mime_part;
 	CamelDataWrapper *dw, *wrapper;
 	CamelMultipart *multipart;
 	CamelInternetAddress *from;
 	CamelStream *stream;
 	CamelContentType *type;
+	GByteArray *byte_array;
 	const gchar *uid;
 	gchar *buffer = NULL, *cid;
 	gchar *from_name = NULL, *from_email = NULL;
@@ -1966,7 +1962,7 @@ build_msg ( ECalBackendExchange *cbex, ECalComponent *comp, const gchar *subject
 	camel_mime_message_set_from (msg, from);
 	g_free (from_name);
 	g_free (from_email);
-	camel_object_unref (from);
+	g_object_unref (from);
 
 	e_cal_component_get_uid (comp, &uid);
 	e_cal_component_get_attachment_list (comp, &attach_list);
@@ -2014,7 +2010,7 @@ build_msg ( ECalBackendExchange *cbex, ECalComponent *comp, const gchar *subject
 		stream = camel_stream_mem_new_with_buffer (file_contents, len);
 		wrapper = camel_data_wrapper_new ();
 		camel_data_wrapper_construct_from_stream (wrapper, stream);
-		camel_object_unref (stream);
+		g_object_unref (stream);
 
 		mime_type = get_mime_type (dest_url);
 		if (mime_type) {
@@ -2026,7 +2022,7 @@ build_msg ( ECalBackendExchange *cbex, ECalComponent *comp, const gchar *subject
 
 		mime_part = camel_mime_part_new ();
 
-		camel_medium_set_content_object (CAMEL_MEDIUM (mime_part), wrapper);
+		camel_medium_set_content (CAMEL_MEDIUM (mime_part), wrapper);
 		camel_mime_part_set_filename (mime_part, mime_filename);
 		camel_mime_part_set_encoding (mime_part, CAMEL_TRANSFER_ENCODING_BASE64);
 		cid = camel_header_msgid_generate ();
@@ -2037,27 +2033,28 @@ build_msg ( ECalBackendExchange *cbex, ECalComponent *comp, const gchar *subject
 		camel_multipart_set_boundary (multipart, NULL);
 		*boundary = g_strdup (camel_multipart_get_boundary (multipart));
 		camel_multipart_add_part (multipart, mime_part);
-		camel_object_unref (mime_part);
+		g_object_unref (mime_part);
 		g_free (cid);
 	}
 	if (!new_attach_list) {
-		camel_object_unref (multipart);
-		camel_object_unref (msg);
+		g_object_unref (multipart);
+		g_object_unref (msg);
 		return NULL;
 	}
 	e_cal_component_set_attachment_list (comp, new_attach_list);
 
-	camel_medium_set_content_object (CAMEL_MEDIUM (msg), CAMEL_DATA_WRAPPER (multipart));
-	camel_object_unref (multipart);
+	camel_medium_set_content (CAMEL_MEDIUM (msg), CAMEL_DATA_WRAPPER (multipart));
+	g_object_unref (multipart);
 
-	content = (CamelStreamMem *)camel_stream_mem_new();
-	dw = camel_medium_get_content_object (CAMEL_MEDIUM (msg));
-	camel_data_wrapper_decode_to_stream(dw, (CamelStream *)content);
-	buffer = g_memdup (content->buffer->data, content->buffer->len);
-	buffer[content->buffer->len] = '\0';
+	byte_array = g_byte_array_new ();
+	stream = camel_stream_mem_new_with_byte_array (byte_array);
+	dw = camel_medium_get_content (CAMEL_MEDIUM (msg));
+	camel_data_wrapper_decode_to_stream (dw, stream);
+	buffer = g_memdup (byte_array->data, byte_array->len);
+	buffer[byte_array->len] = '\0';
 	d(printf ("|| Buffer: \n%s\n", buffer));
-	camel_object_unref (content);
-	camel_object_unref (msg);
+	g_object_unref (stream);
+	g_object_unref (msg);
 
 	return buffer;
 }
