@@ -33,14 +33,13 @@
 #include <glib.h>
 #include <glib/gstdio.h>
 
+#include <camel/camel.h>
 #include <libedataserver/e-source.h>
 #include <libedataserver/e-source-list.h>
 #include <libedataserver/e-source-group.h>
 #include <libedataserver/e-xml-utils.h>
 #include <libedataserver/e-xml-hash-utils.h>
 
-#include <camel/camel-url.h>
-
 #include <e2k-marshal.h>
 #include <e2k-uri.h>
 #include <exchange-account.h>
@@ -736,8 +735,6 @@ exchange_share_config_listener_get_account_for_uri (ExchangeShareConfigListener
 	GSList *accounts, *a;
 	ExchangeAccount *res = NULL;
 
-	g_return_val_if_fail (uri != NULL, NULL);
-
 	if (!excl)
 		excl = exchange_share_config_listener_get_global ();
 
